# functional for executing direct tail recursive functions written in the special form

def tail_recursive_executor(f,*args):
    while True:
        print('calling',f.__name__,'with arguments:',*args)
        base_case, *args = f(*args)
        if base_case:
            return args[0]



def factorial(n : int, acc : int = 1) -> int:
    if n == 0:
        return (True, acc)
    else:
        return (False, n-1, n*acc)

print(tail_recursive_executor(factorial,10),end='\n\n')


def list_sum (l : [int], acc : int = 0) -> int:
    if l == []:
        return (True, acc)
    else:
        return (False, l[1:], acc+l[0])
    
print(tail_recursive_executor(list_sum,[5, 3, 1, 4]),end='\n\n')





# decorator for executing direct tail recursive functions written in the special form

def tail_recursive_executor(f):
    
    def do_as_iteration(*args):
        while True:
            print('calling',f.__name__,'with arguments:',*args)
            base_case, *args = f(*args)
            if base_case:
                return args[0]
            
    return do_as_iteration



@tail_recursive_executor
def factorial(n : int, acc : int = 1) -> int:
    if n == 0:
        return (True, acc)
    else:
        return (False, n-1, n*acc)

print(tail_recursive_executor(factorial(10)),end='\n\n')


@tail_recursive_executor
def list_sum (l : [int], acc : int = 0) -> int:
    if l == []:
        return (True, acc)
    else:
        return (False, l[1:], acc+l[0])

print(list_sum([5, 3, 1, 4]),end='\n\n')
